' Programm: Liest Luxmeter-IC TSL 45315 aus und gibt Werte als Morse- Text
' per NF-Ton aus. Der TSL 45315 wird per I2C- Bus angesteuert.
'
'
' Sprache: Bascom-AVR-Basic, Version 2.0.7.5

' Erluterungen:
'
' Codierung der Morsezeichen: 1 Byte pro Zeichen, niederwertiges Bit zuerst.
' Ist das Bit 0: Morsepunkt (dit)
' Ist das Bit 1: Moersestrich (dah)
' Da Morsezeichen unterschiedlich lang sind, ist das Ende- Zeichen ebenfalls ein 1-Bit.
'
' Beispiel Morsezeichen "a" (dit dah): &B00000110
' Beispiel Morsezeichen "9" (dah dah dah dah dit): &B00101111

' Auslesen der codierten Zeichen:
' niederwertigstes Bit auslesen, Byte um 1 Stelle nach rechts schieben, niederwertigstes Bit
' auslesen. Ist der Gesamtwert des Bytes 1, ist das Zeichen komplett.
'
'
' Timer0 erzeugt den Mithrton. Pin PB0 wird als Timer0-Ausgang OC0A verwendet.
' Port PB3 und PB4 bilden den (Software-) I2C- BUS.
' Stromversorgung des SRF02 erfolgt über Port PB2

' Programm wird duch Bettigen des Resettsters gestartet und fllt nach der Morseausgabe
' in Powerdown

' Version 2a: Powerdown korrigiert, TSL45315 auch in Powerdown geschickt
'--------------------------------------------------------------------

$regfile = "ATtiny13.dat"
$crystal = 1200000                                          ' Taktfrequenz: 1,2 MHz
$hwstack = 32                                               ' falls Interrupts den Stack retten mssen
$swstack = 0                                                ' wird nicht gebraucht
$framesize = 0                                              ' wird nicht gebraucht

Const Dit = 80                                              ' Lnge eines Punkts, daraus ergibt sich die Morsegeschwindigkeit
Const Dah = Dit * 3                                         ' Lnge eines Strichs
Const Wort = Dit * 8                                        ' Lnge der Pause zwischen Wrtern
Const Tonhoehe = 20                                         ' Tonhhe
Const Slaveid = &H52                                        ' I2C-Adresse TSL45315
Const Slave_read = &H53                                     ' Leseadresse 1 hher

Dim I As Byte                                               ' wird in Zhlschleife verwendet
Dim N As Byte                                               ' Aktueller Morsebuchstabe
Dim M As Byte                                               ' wird frs Auslesen der Morsebits verwendet
Dim L As Byte                                               ' niederwertigstes Bit von M verwendet
Dim Msg(12) As Byte                                         ' Feld fr die Morsetext- Bytes
Dim Lob As Byte                                             ' Lowbyte Lichtstrke
Dim Hib As Byte                                             ' Highbyte Lchtstrke
Dim Lux As Word
Dim Number(5) As Byte                                       ' Einzelziffern Lichtstrke
Dim Rest As Word
Dim Stelle As Byte
Dim Anfang As Byte                                          ' wird fr Unterdrckung fhrender Nullen verwendet

Acsr.acd = 1                                                ' Analog-Komparator ausschalten, spart etwas Strom
Didr0 = &B00000111                                          ' Digitaleingnge PB0-PB2 deaktivieren
'
' Morseton mit Timer0 erzeugen:
' Timer0 zhlt das Register TCNT0 hoch. Es wird mit Register OCR0A verglichen.
' Wenn TCNT0 = OCR0A, wird Output OC0A/PB0 umgeschaltet (getoggled)
' Timer0 erzeugt so Dauerton; er wird jedoch nur an den Ausgangspin durchgeschaltet,
' wenn das Datenrichtungs- Register DDRB.0 = 1 ist. Bei 0 wird der Ton unterdrckt.

Config Timer0 = Counter , Prescale = 64 , Compare A = Toggle , Clear Timer = 1
Ocr0a = Tonhoehe

Restore Message                                             ' Data-Feld in das Array "Msg" einlesen
For I = 1 To 12
Read Msg(i)
Next

Waitms 100                                                  ' gegen Prellen des Reset-Tasters
Config Scl = Portb.3                                        ' Ports fuer IIC-Bus
Config Sda = Portb.4

Ddrb.2 = 1
Portb.2 = 1                                                 ' Stromversorgung Breakout-Board ein

Waitms 500                                                  ' Aufwachzeit fr das Breakout-Board
I2cinit
I2cstart
I2cwbyte Slaveid
I2cwbyte &H80                                               ' &H00 AND &H80 -> Command register
I2cwbyte &H03                                               ' normal operation
I2cstop

I2cstart
I2cwbyte Slaveid
I2cwbyte &H84                                               ' &H04 AND &H80 -> Data Lowbyte
Waitms 420                                                  ' Zeitdauer der Messung abwarten
I2cstart
I2cwbyte Slave_read
I2crbyte Lob , Ack
I2crbyte Hib , Ack                                          ' Register wird automatisch beim Lesen inkrementiert -> Data Highbyte
I2cwbyte &H80                                               ' &H00 AND &H80 -> Command register
I2cwbyte &H00                                               ' TSL45315 Powerdown
I2cstop

Lux = Makeint(lob , Hib)                                    ' die beiden Bytes zu 16bit-Zahl zusammenfgen

For Stelle = 1 To 5                                         ' Einzelziffern ins Array Number() schreiben, niedrigste in Number(1)
    Rest = Lux Mod 10                                       ' im Rest (Modulo) von 10 steckt jeweils die letze Stelle der Zahl
    Number(stelle) = Rest
    Lux = Lux / 10                                          ' Zahl um letzte Stelle verringern
Next

For Stelle = 5 To 1 Step -1                                 ' Ziffern ausgeben, hchste zuerst
    If Number(stelle) > 0 And Anfang = 0 Then               ' fhrende Nullen unterdrcken
       Anfang = 1
    End If
    If Anfang = 1 Then
        N = Number(stelle)
        Incr N                                              ' weil Zhhlung der Elemente der Data-Zeilen mit 1 beginnt
        Gosub Morse
    End If
Next
Waitms 500
For N = 11 To 12                                            ' String " lx"
    Gosub Morse
Next

Portb.2 = 0
Powerdown


' Morsezeichenerzeugung:

Morse:

    M = Msg(n)                                              ' Zeichen aus Kette rauspicken
    For I = 1 To 8
       If M = 1 Then                                        ' Das Byte hat nur noch den Wert 1; Zeichenende!
          Goto Sign_end
       End If
        L = M And &B0000001                                 ' niederwertigstes Bit lesen
         Ddrb.0 = 1                                         ' Ton einschalten
       If L = 1 Then
          Waitms Dah                                        ' ist das Bit 1 -> dah
         Else
          Waitms Dit                                        ' ist das Bit 0 -> dit
       End If
         Ddrb.0 = 0                                         ' Ton abschalten
       Waitms Dit                                           ' Pause innerhalb des Morsezeichens
       Shift M , Right , 1                                  ' Bits um eine Stelle nach rechts shiften
    Next I
    Sign_end:
    Waitms Dah                                              ' Pause zwischen Morsezeichen
Return

End


'- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
' Codierung der Morsezeichen. Bei einem Prozessor mit viel SRAM wrde man alle Zeichen
' in ein Array einlesen (bersetzungstabelle), den eigentlichen Bakentext in einen String packen
' und diesen String zeichenenweise abarbeiten.
'
' Da der Attiny13 nur ca. 64 Byte SRAM hat, ist nur Platz fr etwa 50 Bytes. Daher ist der Weg hier
' anders:  Der Bakentext ist ein Array mit max. 52 Elementen, in dem die auszusendenden Zeichen stehen,
' allerdings in einer leicht lesbaren schreibweise.
' Die bersetzungstabelle ist mit Konstanten realisiert. Nur die wirklich bentigten Zeichen werden
' (beim kompilieren des Programmes) umgewandelt.


Const #a = &B00000110
Const #b = &B00010001
Const #c = &B00010101
Const #d = &B00001001
Const #e = &B00000010
Const #f = &B00010100
Const #g = &B00001011
Const #h = &B00010000
Const #i = &B00000100
Const #j = &B00011110
Const #k = &B00001101
Const #l = &B00010010
Const #m = &B00000111
Const #n = &B00000101
Const #o = &B00001111
Const #p = &B00010110
Const #q = &B00011011
Const #r = &B00001010
Const #s = &B00001000
Const #t = &B00000011
Const #u = &B00001100
Const #v = &B00011000
Const #w = &B00001110
Const #x = &B00011001
Const #y = &B00011101
Const #z = &B00010011
Const #ae = &B00011010
Const #oe = &B00010111
Const #ue = &B00011100

Const #0 = &B00111111
Const #1 = &B00111110
Const #2 = &B00111100
Const #3 = &B00111000
Const #4 = &B00110000
Const #5 = &B00100000
Const #6 = &B00100001
Const #7 = &B00100011
Const #8 = &B00100111
Const #9 = &B00101111

Const Leerz = 0
Const Anfuehrz = &B01010010
Const Kl_auf = &B00101101
Const Kl_zu = &B01101101
Const Plus = &B00101010
Const Komma = &B01110011
Const Minus = &B01100001
Const Punkt = &B01101010
Const Slash = &B00101001
Const Dopp_pkt = &B01000111
Const Gleich = &B00110001
Const Fragez = &B01001100



Message:

Data #0 , #1 , #2 , #3 , #4 , #5 , #6 , #7 , #8 , #9
Data #l , #x